"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const TestServer_1 = require("./TestServer");
const decorators_1 = require("./decorators");
const Token_1 = require("./shared/Token");
const fs = require("fs-extra");
const path = require("path");
const superagent = require("superagent");
let StartService = class StartService {
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            this.started = true;
        });
    }
    getAuthDetails(req) {
        const ret = {
            authToken: req.authToken,
        };
        return ret;
    }
    serverStarted(server) {
        this.server = server;
    }
};
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], StartService.prototype, "start", null);
__decorate([
    decorators_1.Get('/auth_details'),
    __param(0, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], StartService.prototype, "getAuthDetails", null);
__decorate([
    decorators_1.ServerStarted(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [TestServer_1.TestServer]),
    __metadata("design:returntype", void 0)
], StartService.prototype, "serverStarted", null);
StartService = __decorate([
    decorators_1.BaseRoute('/'),
    decorators_1.ServiceName('start')
], StartService);
describe('Server', function () {
    let server;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        server = new TestServer_1.TestServer();
    }));
    afterEach(() => __awaiter(this, void 0, void 0, function* () {
        yield server.shutdown().catch(() => { });
    }));
    it('should start by default with no services', function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield server.start();
            chai_1.expect(server.services.length).to.be.eq(0);
        });
    });
    it('should allow skipping autokeygen', function () {
        return __awaiter(this, void 0, void 0, function* () {
            server.config.autoKeyGen = false;
            chai_1.assert.isRejected(server.start());
            chai_1.assert.isFalse(fs.existsSync(path.resolve(server.config.dataPath, "keys", "auth.pub")));
            chai_1.assert.isFalse(fs.existsSync(path.resolve(server.config.dataPath, "keys", "auth.key")));
        });
    });
    it('should be able to add a new service', function () {
        return __awaiter(this, void 0, void 0, function* () {
            server.addService(new StartService());
            yield server.start();
            chai_1.expect(server.services.find(service => service instanceof StartService)).to.be.a('object');
            chai_1.expect(server.services.length).to.be.eq(1);
        });
    });
    it('should be register @ServerStarted decorator', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const startService = new StartService();
            server.addService(startService);
            yield server.start();
            chai_1.expect(server.services.find(service => service instanceof StartService)).to.be.a('object');
            chai_1.expect(server.services.length).to.be.eq(1);
            chai_1.expect(startService.server).to.exist;
            chai_1.expect(startService.server).to.eq(server);
        });
    });
    it('should be able to add an array of services with one method: addServices', function () {
        return __awaiter(this, void 0, void 0, function* () {
            server.addServices(new StartService(), new StartService(), new StartService());
            yield server.start();
            chai_1.expect(server.services.find(service => service instanceof StartService)).to.be.a('object');
            chai_1.expect(server.services.length).to.be.eq(3);
        });
    });
    it('should be able to remove a service by name', function () {
        return __awaiter(this, void 0, void 0, function* () {
            server.addService(new StartService());
            chai_1.expect(server.services.length).to.be.eq(1);
            server.removeService('start');
            yield server.start();
            chai_1.expect(server.services.length).to.be.eq(0);
        });
    });
    it('should be able to remove a service by instance', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const startService = new StartService();
            server.addService(startService);
            chai_1.expect(server.services.length).to.be.eq(1);
            server.removeService(startService);
            yield server.start();
            chai_1.expect(server.services.length).to.be.eq(0);
        });
    });
    describe("authentication", () => {
        let serverUrl;
        beforeEach(() => __awaiter(this, void 0, void 0, function* () {
            server.addService(new StartService());
            yield server.start();
            serverUrl = `http://${server.address}`;
        }));
        describe("Authorization header", () => {
            it("should accept nothing", () => __awaiter(this, void 0, void 0, function* () {
                const r = yield superagent.get(`${serverUrl}/auth_details`);
            }));
            it("should accept a signed token", () => __awaiter(this, void 0, void 0, function* () {
                const refreshToken = new Token_1.RefreshToken({
                    appId: 'io.realm.Test',
                    identity: 'user1',
                    isAdmin: true,
                });
                const r = yield superagent.get(`${serverUrl}/auth_details`)
                    .set('Authorization', refreshToken.sign(server.privateKey));
                chai_1.assert.deepEqual(r.body.authToken, refreshToken.toJSON());
            }));
            it("should throw an error with a malformed token", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isRejected(superagent.get(`${serverUrl}/auth_details`).set('Authorization', "gibberish stuff"), "Unauthorized");
            }));
        });
    });
});
//# sourceMappingURL=Server.spec.js.map