"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("../decorators");
const Server_1 = require("../Server");
const Token_1 = require("../shared/Token");
const realms_1 = require("../realms");
const errors = require("../errors");
const path = require("path");
const moment = require("moment");
const URI = require("urijs");
const _ = require("lodash");
class PermissionService {
    constructor() {
        this.handleManagementRealmChange = (changeEvent) => __awaiter(this, void 0, void 0, function* () {
            const matches = changeEvent.path.match(PermissionService.MANAGEMENT_REALM_REGEX);
            const userId = matches[1];
            const user = this.getUserByUserId(userId);
            const realmPath = changeEvent.path;
            const realm = changeEvent.realm;
            this.logger.debug(`A management realm with path ${changeEvent.path} has changes with userId: ${userId}`);
            if (!user) {
                this.logger.error(`A change notification with management realm of path ${changeEvent.path} 
            was notified with a userId of "${userId}" but this userId was not found in the system. This realm will now be closed.
            Please ensure that ${userId} exists within the __admin Realm. This permission change attempt will be ignored.`);
                realm.close();
                return;
            }
            let unprocessedPermissionOfferObjects = realm.objects("PermissionOffer").filtered('statusCode == null');
            let unprocessedPermissionOfferResponseObjects = realm.objects("PermissionOfferResponse").filtered('statusCode == null');
            let unprocessedPermissionChangeObjects = realm.objects("PermissionChange").filtered('statusCode == null');
            for (let p of unprocessedPermissionOfferObjects) {
                this.handlePermissionOfferObject(p, userId);
            }
            for (let p of unprocessedPermissionOfferResponseObjects) {
                this.handlePermissionOfferResponseObject(p);
            }
            for (let p of unprocessedPermissionChangeObjects) {
                this.handlePermissionChangeObject(p, userId);
            }
        });
    }
    get managementRealm() {
        return this.syncedManagementRealm.realm;
    }
    get adminRealm() {
        return this.syncedAdminRealm.realm;
    }
    serverStarted(server) {
        this.discovery = server.discovery;
        this.server = server;
        this.dataPath = path.resolve(server.config.dataPath, "permissions");
        this.privateKey = server.privateKey;
        this.adminTokenUser = Realm.Sync.User.adminUser(Token_1.generateAdminToken(this.privateKey), "realm://default/__permission");
        this.syncedManagementRealm = server.getSyncedRealm(realms_1.GlobalManagementRealm);
        this.syncedAdminRealm = server.getSyncedRealm(realms_1.AdminRealm);
        const syncBaseUrl = `realm://${server.address}`;
        Realm.Sync.addListener(syncBaseUrl, Realm.Sync.User.adminUser(Token_1.generateAdminToken(this.privateKey), "realm://default/__admin"), PermissionService.MANAGEMENT_REALM_REGEX, 'change', this.handleManagementRealmChange);
        Realm.Sync.addListener(syncBaseUrl, Realm.Sync.User.adminUser(Token_1.generateAdminToken(this.privateKey), "realm://default/__admin"), PermissionService.ADMIN_REALM_REGEX, 'change', this.handleNewUserRegistration);
    }
    stop() {
        Realm.Sync.removeListener(PermissionService.MANAGEMENT_REALM_REGEX, 'change', this.handleManagementRealmChange);
    }
    getUserByUserId(userId) {
        return this.adminRealm.objectForPrimaryKey('User', userId);
    }
    getAllUsers() {
        return this.adminRealm.objects('User');
    }
    getUserIdsWithMetadata(key, value) {
        let metadata = this.adminRealm.objects('UserMetadataRow');
        if (key) {
            metadata = metadata.filtered("key = $0", key);
        }
        if (value) {
            metadata = metadata.filtered("value = $0", value);
        }
        return Array.from(new Set(metadata.map((meta) => meta.user.userId)));
    }
    getUsersByUserIds(userIds) {
        return this.adminRealm.objects("User").filtered('userId IN $0', userIds);
    }
    getManagementRealmForUserId(userId) {
        const realmDefinition = realms_1.CreateUserManagementRealmDefinition(userId);
        const syncedRealm = this.server.getSyncedRealm(realmDefinition);
        return syncedRealm.realm;
    }
    getAdminRealmPermissionForUserAndFile(realmFilePath, userId) {
        const realmFile = this.adminRealm.objectForPrimaryKey("RealmFile", realmFilePath);
        const user = this.getUserByUserId(userId);
        if (!realmFile) {
            throw new errors.realm.InvalidParameters('realmFilePath does not exist');
        }
        const permission = realmFile.permissions.filtered('userId = $0', userId)[0];
        if (!permission) {
            this.adminRealm.write(() => {
                realmFile.permissions.push({
                    user: user,
                    updatedAt: new Date(),
                    mayManage: false,
                    mayRead: false,
                    mayWrite: false
                });
            });
        }
        return {
            permission: permission,
            adminRealm: this.adminRealm
        };
    }
    getUserSpecificPermissionAndRealm(realmPath, userId) {
        const realmDefinition = realms_1.CreateUserPermissionRealmDefinition(userId);
        const synedUserRealm = this.server.getSyncedRealm(realmDefinition);
        const permission = synedUserRealm.realm.objectForPrimaryKey("Permission", realmPath);
        return {
            userSpecificRealm: synedUserRealm.realm,
            permission: permission
        };
    }
    handlePermissionOfferObject(permissionOffer, userId) {
        if (permissionOffer.realmUrl === '') {
            throw new errors.realm.InvalidParameters({
                name: 'realmUrl',
                reason: 'The realmUrl cannot be empty and must encode a path'
            });
        }
        const realmPath = new URI(permissionOffer.realmUrl).path();
        if (_.includes(realmPath, '__management')) {
            throw new errors.realm.InvalidParameters({
                name: 'realmUrl',
                reason: 'The realmUrl cannot be of a management realm'
            });
        }
        const realmFile = this.adminRealm.objectForPrimaryKey("RealmFile", realmPath);
        if (!realmFile) {
            throw new errors.realm.InvalidParameters({
                name: 'realmUrl',
                reason: 'The realmUrl does not exist'
            });
        }
        if (permissionOffer.expiresAt && moment(permissionOffer.expiresAt).isBefore(moment.now())) {
            throw new errors.realm.ExpiredPermissionOffer({
                title: `The permission offer has expired on ${permissionOffer.expiresAt}. The current time is ${new Date()}.`
            });
        }
        this.adminRealm.write(() => {
            permissionOffer.token = `${userId}:${permissionOffer.id}`;
        });
    }
    handlePermissionOfferResponseObject(permissionOfferResponse) {
        const tokenComponents = permissionOfferResponse.token.split(':');
        if (tokenComponents.length < 2 || tokenComponents[0].length === 0) {
            throw new errors.realm.InvalidParameters('token');
        }
        const offeringUserId = tokenComponents[0];
        const offeringRealm = this.getManagementRealmForUserId(offeringUserId);
        const permissionOffers = offeringRealm.objects("PermissionOffer").filtered('token = %0', permissionOfferResponse.token);
        if (permissionOffers.length === 0) {
            throw new errors.realm.InvalidParameters('token');
        }
        if (permissionOffers.length > 1) {
            throw new errors.realm.AmbiguousPermissionOfferToken({
                detail: `Unexpected count of permission offers (=${permissionOffers.length})`
                    + ` matching the token '${permissionOfferResponse.token}' of the permission request!`,
            });
        }
        const permissionOffer = permissionOffers[0];
        permissionOfferResponse.realmUrl = permissionOffer.realmUrl;
        permissionOfferResponse.token = permissionOffer.token;
        const realmPath = new URI(permissionOfferResponse.realmUrl).path();
        const { permission, adminRealm } = this.getAdminRealmPermissionForUserAndFile(realmPath, offeringUserId);
        adminRealm.write(() => {
            permission.mayWrite = permissionOffer.mayWrite;
            permission.mayRead = permission.mayRead;
            permission.mayManage = permission.mayManage;
        });
    }
    handlePermissionChangeObject(permissionChange, userId) {
        const realmPath = new URI(permissionChange.realmUrl).path();
        let realmFiles;
        if (permissionChange.realmUrl !== '*') {
            realmFiles = this.adminRealm.objects("RealmFile")
                .filtered('path = $0', path);
        }
        let users;
        if (permissionChange.metadataKey) {
            const userIds = this.getUserIdsWithMetadata(permissionChange.metadataKey, permissionChange.metadataValue);
            this.logger.debug(`Apply permission change by metadata query 
                (${permissionChange.metadataKey}=${permissionChange.metadataValue}) 
                matching the userIds ${userIds}`);
            users = this.getUsersByUserIds(userIds);
        }
        else if (permissionChange.userId === '*') {
            this.logger.debug(`Apply permission change for all users *`);
            users = this.getAllUsers();
        }
        else {
            this.logger.debug(`Apply permission change for all users ${permissionChange.userId}`);
            users = this.getUsersByUserIds([permissionChange.userId]);
        }
        if (!realmFiles) {
            realmFiles = this.adminRealm
                .objects("RealmFile")
                .filtered('creatorUserId = $0', userId);
        }
        this.setPermissionsForEntireSystem(users, realmFiles, permissionChange.mayWrite, permissionChange.mayRead, permissionChange.mayManage);
    }
    handleNewUserRegistration(changeEvent) {
    }
    setPermissionsForEntireSystem(users, realmFiles, mayWrite, mayRead, mayManage, permissionChange) {
        for (let file of realmFiles) {
            for (let user of users) {
                {
                    const { adminRealm, permission } = this.getAdminRealmPermissionForUserAndFile(file.path, user.userId);
                    adminRealm.write(() => {
                        permission.mayManage = mayManage;
                        permission.mayRead = mayRead;
                        permission.mayWrite = mayWrite;
                        permission.updatedAt = new Date();
                    });
                }
                {
                    const { userSpecificRealm, permission } = this.getUserSpecificPermissionAndRealm(file.path, user.userId);
                    userSpecificRealm.write(() => {
                        userSpecificRealm.create("UserSpecificPermission", {
                            userId: user.userId,
                            path: file.path,
                            mayManage: mayManage,
                            mayWrite: mayWrite,
                            mayRead: mayRead,
                            updatedAt: new Date()
                        }, true);
                    });
                }
            }
        }
        if (permissionChange) {
            permissionChange.statusMessage = '';
            permissionChange.statusCode;
        }
    }
}
PermissionService.MANAGEMENT_REALM_REGEX = '^/([^/]+)/__management$';
PermissionService.ADMIN_REALM_REGEX = `/^__admin$/`;
__decorate([
    decorators_1.ServerStarted(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Server_1.Server]),
    __metadata("design:returntype", void 0)
], PermissionService.prototype, "serverStarted", null);
__decorate([
    decorators_1.Stop(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PermissionService.prototype, "stop", null);
exports.PermissionService = PermissionService;
//# sourceMappingURL=PermissionsService.js.map