"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("../decorators");
const Logger_1 = require("../shared/Logger");
const Token_1 = require("../shared/Token");
const Server_1 = require("../Server");
const errors = require("../errors");
const realms_1 = require("../realms");
const path = require("path");
const URI = require("urijs");
let RealmDirectoryService = class RealmDirectoryService {
    constructor(config = {}) {
        this.logger = new Logger_1.Mute();
        this.config = config;
        if (!this.config.tags || this.config.tags.length === 0) {
            this.config.tags = ["master"];
        }
        if (!this.privateKey) {
            this.privateKey = path.resolve(process.cwd(), 'data/keys/auth.key');
        }
        this.tags = this.config.tags;
    }
    get adminRealm() {
        return this.syncedAdminRealm.realm;
    }
    setLogger(l) {
        this.logger = l;
    }
    find() {
        return __awaiter(this, void 0, void 0, function* () {
            const realmFiles = this.adminRealm.objects("RealmFile").slice();
            return realmFiles;
        });
    }
    findByPath(realmPath, accessToken) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!accessToken) {
                throw new errors.JSONError({
                    title: "X-Realm-Access-Token header is not valid",
                    status: 400,
                });
            }
            const token = Token_1.Token.parse(accessToken, this.publicKey);
            if (realmPath.includes('~')) {
                throw new errors.realm.InvalidParameters({
                    name: 'realmPath',
                    reason: `realmPath cannot contain a ~. Please use a full absolute path when trying to find the realmFile.`
                });
            }
            let realmFile = this.adminRealm.objectForPrimaryKey("RealmFile", realmPath);
            if (!realmFile) {
                realmFile = (yield this.createRealmFile(realmPath, token.identity));
                const user = this.adminRealm.objectForPrimaryKey('User', token.identity);
                const isHomeDirectoryOwner = new URI(realmPath).segment()[0] === token.identity;
                const isUserAnAdminOrAdminTokenUser = token.identity === '__admin' || user.isAdmin === true;
                if (!isUserAnAdminOrAdminTokenUser && isHomeDirectoryOwner) {
                    this.adminRealm.write(() => {
                        realmFile.permissions.push({
                            mayManage: true,
                            mayRead: true,
                            mayWrite: true,
                            updatedAt: new Date(),
                            user: user
                        });
                    });
                }
            }
            return realmFile;
        });
    }
    remove(realmPath, accessToken) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!accessToken) {
                throw new errors.JSONError({
                    title: "X-Realm-Access-Token header is not valid",
                    status: 400,
                });
            }
            const token = Token_1.Token.parse(accessToken, this.publicKey);
            realmPath = realmPath.replace('~', token.identity);
            const realmFile = this.adminRealm.objectForPrimaryKey("RealmFile", realmPath);
            if (realmFile) {
                this.adminRealm.write(() => this.adminRealm.delete(realmFile));
            }
            return {};
        });
    }
    validatedRealmFile(path) {
        if (!path) {
            throw new errors.realm.MissingParameters('path');
        }
        let realmFile = this.adminRealm.objectForPrimaryKey('RealmFile', path);
        if (!realmFile) {
            throw new errors.JSONError({
                detail: `Realm path: ${path}`,
                title: 'The provided path does not exist',
                status: 403
            });
        }
        return realmFile;
    }
    start(server) {
        return __awaiter(this, void 0, void 0, function* () {
            this.discovery = server.discovery;
            this.privateKey = server.privateKey;
            this.publicKey = server.publicKey;
            this.syncedAdminRealm = server.getSyncedRealm(realms_1.AdminRealm);
            try {
                let adminRealmPath = '/__admin';
                let adminRealmFile = this.adminRealm.objectForPrimaryKey('RealmFile', adminRealmPath);
                if (!adminRealmFile) {
                    this.createRealmFile(adminRealmPath, '', "default");
                }
                let permissionsRealmPath = '/__permissions';
                let permissionsRealmFile = this.adminRealm.objectForPrimaryKey('RealmFile', permissionsRealmPath);
                if (!permissionsRealmFile) {
                    this.createRealmFile(permissionsRealmPath, '', "default");
                }
            }
            catch (err) {
                this.logger.info('Failed to create directory entries for internal Realms', this);
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.syncedAdminRealm) {
                this.syncedAdminRealm.close();
            }
        });
    }
    createRealmFile(path, creatorId, syncLabel) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!syncLabel) {
                const sync = yield this.discovery.find('sync', ["master"]);
                if (!sync) {
                    throw new Error("Could not find a sync worker for allocation");
                }
                syncLabel = sync.tags.filter((t) => {
                    return t !== "master" && t !== "slave";
                })[0] || "default";
            }
            let realmFile;
            this.adminRealm.write(() => {
                realmFile = this.adminRealm.create('RealmFile', {
                    creatorId,
                    path,
                    creationDate: new Date(),
                    syncLabel: syncLabel,
                });
            });
            return realmFile;
        });
    }
};
__decorate([
    decorators_1.Unmute(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Logger_1.Logger]),
    __metadata("design:returntype", void 0)
], RealmDirectoryService.prototype, "setLogger", null);
__decorate([
    decorators_1.Get('/files'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RealmDirectoryService.prototype, "find", null);
__decorate([
    decorators_1.Get('/files/:realmPath'),
    __param(0, decorators_1.Params('realmPath')),
    __param(1, decorators_1.Headers('X-Realm-Access-Token')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], RealmDirectoryService.prototype, "findByPath", null);
__decorate([
    decorators_1.Delete('/files/:realmPath'),
    __param(0, decorators_1.Params('realmPath')),
    __param(1, decorators_1.Headers('X-Realm-Access-Token')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], RealmDirectoryService.prototype, "remove", null);
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Server_1.Server]),
    __metadata("design:returntype", Promise)
], RealmDirectoryService.prototype, "start", null);
__decorate([
    decorators_1.Stop(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RealmDirectoryService.prototype, "stop", null);
RealmDirectoryService = __decorate([
    decorators_1.BaseRoute('/realms'),
    decorators_1.ServiceName('realms'),
    __metadata("design:paramtypes", [Object])
], RealmDirectoryService);
exports.RealmDirectoryService = RealmDirectoryService;
//# sourceMappingURL=RealmDirectoryService.js.map